Package: mlr3shiny
Title: Machine Learning in 'shiny' with 'mlr3'
Version: 0.5.0
Authors@R: 
    c(person(given = "Laurens",
           family = "Tetzlaff",
           role = c("aut", "cre"),
           email = "laurens.tetzlaff@outlook.de"),
     person(given = "Gero",
            family = "Szepannek",
            role = "aut",
            email = "gero.szepannek@web.de"),
     person(given = "Alexander",
            family = "Frahm",
            role = "aut"),
     person(given = "Dominic",
            family = "Henning",
            role = "aut",
            email  = "henning.dominic1705@gmail.com"),
	 person(given = "Jakob",
	 		family = "Steinbrink",
			role = "aut")
		)
Description: A web-based graphical user interface to provide the basic steps of a machine learning workflow. It uses the functionalities of the 'mlr3' framework.
License: BSD_2_clause + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.6)
Imports: mlr3 (>= 0.12.0), mlr3measures (>= 0.3.1), mlr3learners,
        mlr3pipelines, mlr3viz, Metrics, shiny (>= 1.6.0),
        shinydashboard, shinyjs (>= 2.0.0), shinyWidgets, shinyalert,
        data.table (>= 1.12.8), DT (>= 0.11), stringr, plyr, dplyr,
        purrr, stats, patchwork, ggparty, GGally
RoxygenNote: 7.3.2
Suggests: testthat (>= 2.1.0), shinytest, devtools, ranger, e1071,
        xgboost, igraph, readxl, DALEX, DALEXtra, bslib, haven
NeedsCompilation: no
Packaged: 2024-12-28 13:04:27 UTC; laurensmartintetzlaff
Author: Laurens Tetzlaff [aut, cre],
  Gero Szepannek [aut],
  Alexander Frahm [aut],
  Dominic Henning [aut],
  Jakob Steinbrink [aut]
Maintainer: Laurens Tetzlaff <laurens.tetzlaff@outlook.de>
Repository: CRAN
Date/Publication: 2025-01-09 13:50:02 UTC
Built: R 4.6.0; ; 2026-04-30 13:52:01 UTC; unix
